@@ -1,9 +1,11 @@
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
+
 def setup_module(module):
     module.TestStateFullThing.classcount = 0
 
 
-class TestStateFullThing(object):
+class TestStateFullThing:
     def setup_class(cls):
         cls.classcount += 1
 
