@@ -1,8 +1,10 @@
-# -*- coding: utf-8 -*-
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import pytest
 
 
-class TestClass(object):
+class TestClass:
     @pytest.fixture
     def something(self, request):
         return request.instance
