@@ -1,14 +1,12 @@
 # mypy: allow-untyped-defs
+from __future__ import annotations
+
+from collections.abc import Generator
+from collections.abc import Sequence
 import os
 from pathlib import Path
 import textwrap
 from typing import cast
-from typing import Dict
-from typing import Generator
-from typing import List
-from typing import Optional
-from typing import Sequence
-from typing import Union
 
 from _pytest.config import ExitCode
 from _pytest.config import PytestPluginManager
@@ -27,8 +25,8 @@ def ConftestWithSetinitial(path) -> PytestPluginManager:
 
 def conftest_setinitial(
     conftest: PytestPluginManager,
-    args: Sequence[Union[str, Path]],
-    confcutdir: Optional[Path] = None,
+    args: Sequence[str | Path],
+    confcutdir: Path | None = None,
 ) -> None:
     conftest._set_initial_conftests(
         args=args,
@@ -45,9 +43,7 @@ def conftest_setinitial(
 @pytest.mark.usefixtures("_sys_snapshot")
 class TestConftestValueAccessGlobal:
     @pytest.fixture(scope="module", params=["global", "inpackage"])
-    def basedir(
-        self, request, tmp_path_factory: TempPathFactory
-    ) -> Generator[Path, None, None]:
+    def basedir(self, request, tmp_path_factory: TempPathFactory) -> Generator[Path]:
         tmp_path = tmp_path_factory.mktemp("basedir", numbered=True)
         tmp_path.joinpath("adir/b").mkdir(parents=True)
         tmp_path.joinpath("adir/conftest.py").write_text(
@@ -70,7 +66,7 @@ class TestConftestValueAccessGlobal:
         )
         assert conftest._rget_with_confmod("a", p)[1] == 1
 
-    def test_immediate_initialiation_and_incremental_are_the_same(
+    def test_immediate_initialization_and_incremental_are_the_same(
         self, basedir: Path
     ) -> None:
         conftest = PytestPluginManager()
@@ -280,7 +276,7 @@ def test_conftest_confcutdir(pytester: Pytester) -> None:
         ),
         encoding="utf-8",
     )
-    result = pytester.runpytest("-h", "--confcutdir=%s" % x, x)
+    result = pytester.runpytest("-h", f"--confcutdir={x}", x)
     result.stdout.fnmatch_lines(["*--xyz*"])
     result.stdout.no_fnmatch_line("*warning: could not load initial*")
 
@@ -380,7 +376,7 @@ def test_conftest_symlink_files(pytester: Pytester) -> None:
             """
         ),
     }
-    pytester.makepyfile(**{"real/%s" % k: v for k, v in source.items()})
+    pytester.makepyfile(**{f"real/{k}": v for k, v in source.items()})
 
     # Create a build directory that contains symlinks to actual files
     # but doesn't symlink actual directories.
@@ -396,13 +392,13 @@ def test_conftest_symlink_files(pytester: Pytester) -> None:
 
 @pytest.mark.skipif(
     os.path.normcase("x") != os.path.normcase("X"),
-    reason="only relevant for case insensitive file systems",
+    reason="only relevant for case-insensitive file systems",
 )
 def test_conftest_badcase(pytester: Pytester) -> None:
     """Check conftest.py loading when directory casing is wrong (#5792)."""
     pytester.path.joinpath("JenkinsRoot/test").mkdir(parents=True)
     source = {"setup.py": "", "test/__init__.py": "", "test/conftest.py": ""}
-    pytester.makepyfile(**{"JenkinsRoot/%s" % k: v for k, v in source.items()})
+    pytester.makepyfile(**{f"JenkinsRoot/{k}": v for k, v in source.items()})
 
     os.chdir(pytester.path.joinpath("jenkinsroot/test"))
     result = pytester.runpytest()
@@ -462,7 +458,7 @@ def test_conftest_import_order(pytester: Pytester, monkeypatch: MonkeyPatch) ->
         rootpath=pytester.path,
         consider_namespace_packages=False,
     )
-    mods = cast(List[Path], conftest._getconftestmodules(sub))
+    mods = cast(list[Path], conftest._getconftestmodules(sub))
     expected = [ct1, ct2]
     assert mods == expected
 
@@ -536,7 +532,7 @@ def test_conftest_found_with_double_dash(pytester: Pytester) -> None:
 
 
 class TestConftestVisibility:
-    def _setup_tree(self, pytester: Pytester) -> Dict[str, Path]:  # for issue616
+    def _setup_tree(self, pytester: Pytester) -> dict[str, Path]:  # for issue616
         # example mostly taken from:
         # https://mail.python.org/pipermail/pytest-dev/2014-September/002617.html
         runner = pytester.mkdir("empty")
@@ -638,9 +634,9 @@ class TestConftestVisibility:
     ) -> None:
         """#616"""
         dirs = self._setup_tree(pytester)
-        print("pytest run in cwd: %s" % (dirs[chdir].relative_to(pytester.path)))
-        print("pytestarg        : %s" % testarg)
-        print("expected pass    : %s" % expect_ntests_passed)
+        print(f"pytest run in cwd: {dirs[chdir].relative_to(pytester.path)}")
+        print(f"pytestarg        : {testarg}")
+        print(f"expected pass    : {expect_ntests_passed}")
         os.chdir(dirs[chdir])
         reprec = pytester.inline_run(
             testarg,
@@ -658,7 +654,7 @@ class TestConftestVisibility:
 def test_search_conftest_up_to_inifile(
     pytester: Pytester, confcutdir: str, passed: int, error: int
 ) -> None:
-    """Test that conftest files are detected only up to an ini file, unless
+    """Test that conftest files are detected only up to a configuration file, unless
     an explicit --confcutdir option is given.
     """
     root = pytester.path
@@ -699,13 +695,13 @@ def test_search_conftest_up_to_inifile(
 
     args = [str(src)]
     if confcutdir:
-        args = ["--confcutdir=%s" % root.joinpath(confcutdir)]
+        args = [f"--confcutdir={root.joinpath(confcutdir)}"]
     result = pytester.runpytest(*args)
     match = ""
     if passed:
-        match += "*%d passed*" % passed
+        match += f"*{passed} passed*"
     if error:
-        match += "*%d error*" % error
+        match += f"*{error} error*"
     result.stdout.fnmatch_lines(match)
 
 
