@@ -1,11 +1,11 @@
 # mypy: allow-untyped-defs
 from __future__ import annotations
 
+from collections.abc import Callable
 import inspect
 from pathlib import Path
 import sys
 import textwrap
-from typing import Callable
 
 from _pytest.doctest import _get_checker
 from _pytest.doctest import _is_main_py
@@ -33,24 +33,24 @@ class TestDoctests:
 
         for x in (pytester.path, checkfile):
             # print "checking that %s returns custom items" % (x,)
-            items, reprec = pytester.inline_genitems(x)
+            items, _reprec = pytester.inline_genitems(x)
             assert len(items) == 1
             assert isinstance(items[0], DoctestItem)
             assert isinstance(items[0].parent, DoctestTextfile)
         # Empty file has no items.
-        items, reprec = pytester.inline_genitems(w)
+        items, _reprec = pytester.inline_genitems(w)
         assert len(items) == 0
 
     def test_collect_module_empty(self, pytester: Pytester):
         path = pytester.makepyfile(whatever="#")
         for p in (path, pytester.path):
-            items, reprec = pytester.inline_genitems(p, "--doctest-modules")
+            items, _reprec = pytester.inline_genitems(p, "--doctest-modules")
             assert len(items) == 0
 
     def test_collect_module_single_modulelevel_doctest(self, pytester: Pytester):
         path = pytester.makepyfile(whatever='""">>> pass"""')
         for p in (path, pytester.path):
-            items, reprec = pytester.inline_genitems(p, "--doctest-modules")
+            items, _reprec = pytester.inline_genitems(p, "--doctest-modules")
             assert len(items) == 1
             assert isinstance(items[0], DoctestItem)
             assert isinstance(items[0].parent, DoctestModule)
@@ -64,7 +64,7 @@ class TestDoctests:
         """
         )
         for p in (path, pytester.path):
-            items, reprec = pytester.inline_genitems(p, "--doctest-modules")
+            items, _reprec = pytester.inline_genitems(p, "--doctest-modules")
             assert len(items) == 2
             assert isinstance(items[0], DoctestItem)
             assert isinstance(items[1], DoctestItem)
@@ -97,7 +97,7 @@ class TestDoctests:
             },
         )
         for p in (path, pytester.path):
-            items, reprec = pytester.inline_genitems(p, "--doctest-modules")
+            items, _reprec = pytester.inline_genitems(p, "--doctest-modules")
             assert len(items) == 2
             assert isinstance(items[0], DoctestItem)
             assert isinstance(items[1], DoctestItem)
@@ -223,9 +223,12 @@ class TestDoctests:
                 "002 >>> 0 / i",
                 "UNEXPECTED EXCEPTION: ZeroDivisionError*",
                 "Traceback (most recent call last):",
-                '  File "*/doctest.py", line *, in __run',
-                "    *",
-                *((" *^^^^*", " *", " *") if sys.version_info >= (3, 13) else ()),
+                *(
+                    ('  File "*/doctest.py", line *, in __run', "    *")
+                    if sys.version_info <= (3, 14)
+                    else ()
+                ),
+                *((" *^^^^*", " *", " *") if sys.version_info[:2] == (3, 13) else ()),
                 '  File "<doctest test_doctest_unexpected_exception.txt[1]>", line 1, in <module>',
                 "ZeroDivisionError: division by zero",
                 "*/test_doctest_unexpected_exception.txt:2: UnexpectedException",
@@ -837,7 +840,7 @@ class TestDoctests:
                 return 'c'
         """
         )
-        items, reprec = pytester.inline_genitems(p, "--doctest-modules")
+        items, _reprec = pytester.inline_genitems(p, "--doctest-modules")
         reportinfo = items[0].reportinfo()
         assert reportinfo[1] == 1
 
@@ -904,7 +907,7 @@ class TestLiterals:
     def test_allow_unicode(self, pytester, config_mode):
         """Test that doctests which output unicode work in all python versions
         tested by pytest when the ALLOW_UNICODE option is used (either in
-        the ini file or by an inline comment).
+        the configuration file or by an inline comment).
         """
         if config_mode == "ini":
             pytester.makeini(
@@ -939,7 +942,7 @@ class TestLiterals:
     def test_allow_bytes(self, pytester, config_mode):
         """Test that doctests which output bytes work in all python versions
         tested by pytest when the ALLOW_BYTES option is used (either in
-        the ini file or by an inline comment)(#1287).
+        the configuration file or by an inline comment)(#1287).
         """
         if config_mode == "ini":
             pytester.makeini(
@@ -1328,7 +1331,7 @@ class TestDoctestAutoUseFixtures:
         params = ("--doctest-modules",) if enable_doctest else ()
         passes = 3 if enable_doctest else 2
         result = pytester.runpytest(*params)
-        result.stdout.fnmatch_lines(["*=== %d passed in *" % passes])
+        result.stdout.fnmatch_lines([f"*=== {passes} passed in *"])
 
     @pytest.mark.parametrize("scope", SCOPES)
     @pytest.mark.parametrize("autouse", [True, False])
@@ -1593,7 +1596,14 @@ class Broken:
 
 
 @pytest.mark.parametrize(  # pragma: no branch (lambdas are not called)
-    "stop", [None, _is_mocked, lambda f: None, lambda f: False, lambda f: True]
+    "stop",
+    [
+        None,
+        pytest.param(_is_mocked, id="is_mocked"),
+        pytest.param(lambda f: None, id="lambda_none"),
+        pytest.param(lambda f: False, id="lambda_false"),
+        pytest.param(lambda f: True, id="lambda_true"),
+    ],
 )
 def test_warning_on_unwrap_of_broken_object(
     stop: Callable[[object], object] | None,
