@@ -1,6 +1,7 @@
-from typing import List
-from typing import Optional
-from typing import Type
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
+import sys
 import warnings
 
 import pytest
@@ -51,7 +52,7 @@ class TestSubclassWarningPop:
         pass
 
     @staticmethod
-    def raise_warnings_from_list(_warnings: List[Type[Warning]]):
+    def raise_warnings_from_list(_warnings: list[type[Warning]]):
         for warn in _warnings:
             warnings.warn(f"Warning {warn().__repr__()}", warn)
 
@@ -131,7 +132,7 @@ class TestWarningsRecorderChecker:
 class TestDeprecatedCall:
     """test pytest.deprecated_call()"""
 
-    def dep(self, i: int, j: Optional[int] = None) -> int:
+    def dep(self, i: int, j: int | None = None) -> int:
         if i == 0:
             warnings.warn("is deprecated", DeprecationWarning, stacklevel=1)
         return 42
@@ -228,7 +229,7 @@ class TestDeprecatedCall:
         for warning in other_warnings:
 
             def f():
-                warnings.warn(warning("hi"))
+                warnings.warn(warning("hi"))  # noqa: B023
 
             with pytest.warns(warning):
                 with pytest.raises(pytest.fail.Exception):
@@ -346,17 +347,9 @@ class TestWarns:
         assert str(record[0].message) == "user"
         assert str(record[1].message) == "runtime"
 
-    def test_record_only_none_deprecated_warn(self) -> None:
-        # This should become an error when WARNS_NONE_ARG is removed in Pytest 8.0
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            with pytest.warns(None) as record:  # type: ignore[call-overload]
-                warnings.warn("user", UserWarning)
-                warnings.warn("runtime", RuntimeWarning)
-
-            assert len(record) == 2
-            assert str(record[0].message) == "user"
-            assert str(record[1].message) == "runtime"
+    def test_record_only_none_type_error(self) -> None:
+        with pytest.raises(TypeError):
+            pytest.warns(None)  # type: ignore[call-overload]
 
     def test_record_by_subclass(self) -> None:
         with pytest.warns(Warning) as record:
@@ -527,7 +520,7 @@ class TestWarns:
 
             def test_it():
                 with pytest.warns(Warning):
-                    pytest.exit("BOOM")
+                    pytest.exit()
             """,
         )
 
@@ -552,6 +545,42 @@ class TestWarns:
         result.assert_outcomes()
 
 
+def test_raise_type_error_on_invalid_warning() -> None:
+    """Check pytest.warns validates warning messages are strings (#10865) or
+    Warning instances (#11959)."""
+    with pytest.raises(TypeError, match="Warning must be str or Warning"):
+        with pytest.warns(UserWarning):
+            warnings.warn(1)  # type: ignore
+
+
+@pytest.mark.parametrize(
+    "message",
+    [
+        pytest.param("Warning", id="str"),
+        pytest.param(UserWarning(), id="UserWarning"),
+        pytest.param(Warning(), id="Warning"),
+    ],
+)
+def test_no_raise_type_error_on_valid_warning(message: str | Warning) -> None:
+    """Check pytest.warns validates warning messages are strings (#10865) or
+    Warning instances (#11959)."""
+    with pytest.warns(Warning):
+        warnings.warn(message)
+
+
+@pytest.mark.skipif(
+    hasattr(sys, "pypy_version_info"),
+    reason="Not for pypy",
+)
+def test_raise_type_error_on_invalid_warning_message_cpython() -> None:
+    # Check that we get the same behavior with the stdlib, at least if filtering
+    # (see https://github.com/python/cpython/issues/103577 for details)
+    with pytest.raises(TypeError):
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", "test")
+            warnings.warn(1)  # type: ignore
+
+
 def test_multiple_arg_custom_warning() -> None:
     """Test for issue #11906."""
 
