@@ -1,8 +1,11 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import sys
 
-import pytest
 from _pytest.config import ExitCode
 from _pytest.pytester import Pytester
+import pytest
 
 
 @pytest.fixture(params=["--setup-only", "--setup-plan", "--setup-show"], scope="module")
