@@ -280,8 +280,7 @@ def test_warning_recorded_hook(pytester: Pytester) -> None:
         ("call warning", "runtest", "test_warning_recorded_hook.py::test_func"),
         ("teardown warning", "runtest", "test_warning_recorded_hook.py::test_func"),
     ]
-    assert len(collected) == len(expected)  # python < 3.10 zip(strict=True)
-    for collected_result, expected_result in zip(collected, expected):
+    for collected_result, expected_result in zip(collected, expected, strict=True):
         assert collected_result[0] == expected_result[0], str(collected)
         assert collected_result[1] == expected_result[1], str(collected)
         assert collected_result[2] == expected_result[2], str(collected)
@@ -383,7 +382,7 @@ def test_hide_pytest_internal_warnings(
 def test_option_precedence_cmdline_over_ini(
     pytester: Pytester, ignore_on_cmdline
 ) -> None:
-    """Filters defined in the command-line should take precedence over filters in ini files (#3946)."""
+    """Filters defined in the command-line should take precedence over filters in config files (#3946)."""
     pytester.makeini(
         """
         [pytest]
@@ -425,6 +424,33 @@ def test_option_precedence_mark(pytester: Pytester) -> None:
     result.stdout.fnmatch_lines(["* 1 failed in*"])
 
 
+def test_accept_unknown_category(pytester: Pytester) -> None:
+    """Category types that can't be imported don't cause failure (#13732)."""
+    pytester.makeini(
+        """
+        [pytest]
+        filterwarnings =
+            always:Failed to import filter module.*:pytest.PytestConfigWarning
+            ignore::foobar.Foobar
+    """
+    )
+    pytester.makepyfile(
+        """
+        def test():
+            pass
+    """
+    )
+    result = pytester.runpytest_subprocess("-W", "ignore::bizbaz.Bizbaz")
+    result.stdout.fnmatch_lines(
+        [
+            f"*== {WARNINGS_SUMMARY_HEADER} ==*",
+            "*PytestConfigWarning: Failed to import filter module 'foobar': ignore::foobar.Foobar",
+            "*PytestConfigWarning: Failed to import filter module 'bizbaz': ignore::bizbaz.Bizbaz",
+            "* 1 passed, * warning*",
+        ]
+    )
+
+
 class TestDeprecationWarningsByDefault:
     """
     Note: all pytest runs are executed in a subprocess so we don't inherit warning filters
@@ -536,7 +562,8 @@ class TestDeprecationWarningsByDefault:
         )
 
 
-@pytest.mark.skip("not relevant until pytest 9.0")
+# In 9.1, uncomment below and change RemovedIn9 -> RemovedIn10.
+# @pytest.mark.skip("not relevant until pytest 10.0")
 @pytest.mark.parametrize("change_default", [None, "ini", "cmdline"])
 def test_removed_in_x_warning_as_error(pytester: Pytester, change_default) -> None:
     """This ensures that PytestRemovedInXWarnings raised by pytest are turned into errors.
@@ -715,10 +742,8 @@ class TestStackLevel:
         assert func == "<module>"  # the above conftest.py
         assert lineno == 4
 
-    def test_issue4445_preparse(self, pytester: Pytester, capwarn) -> None:
-        """#4445: Make sure the warning points to a reasonable location
-        See origin of _issue_warning_captured at: _pytest.config.__init__.py:910
-        """
+    def test_issue4445_initial_conftest(self, pytester: Pytester, capwarn) -> None:
+        """#4445: Make sure the warning points to a reasonable location."""
         pytester.makeconftest(
             """
             import nothing
@@ -734,7 +759,7 @@ class TestStackLevel:
 
         assert "could not load initial conftests" in str(warning.message)
         assert f"config{os.sep}__init__.py" in file
-        assert func == "_preparse"
+        assert func == "parse"
 
     @pytest.mark.filterwarnings("default")
     def test_conftest_warning_captured(self, pytester: Pytester) -> None:
